#!/bin/sh
# Скрипт выполняет:
# Включает электрические цепи модема
# Ищет usb устройство соответствующее шаблону
# Если есть драйвер устройства с названием VendorID:ProductID создает на него ссылку
# Запускает скрипт соединения в драйвере модема
#
# Требования к скрипту драйвера модема:
# Должен настроить модем
# Должен запустить pppd & и завершиться (не ждать завершения pppd)

# Переменные для конфигурирования
chargePinName="GSM_CHG"             # имя пина заряда
gsmOnPinName="GSM_ON"               # имя пина включения стабилизатора питания
pwrkeyPinName="PWR_KEY"             # имя пина включения модема
gsmRstPinName="GSM_RST"             # имя пина сброса модема
gsmStatusPinName="GSM_STATUS"       # имя пина статуса модема
pattern="A76XX"                     # Шаблон для поиска устройств в выводе lsusb, разделять шаблоны символом |
parentDirectory="/run/ppp"          # Папка для создания ссылок
distanationLinkName="/peers"        # Папка в которой создается ссылка на драйвер модема
driversDirectory="/etc/ppp/peers"   # Папка где должны быть расположены драйвера модема
pppdPidFile="/run/ppp0.pid"         # Файл процесса pppd
startScriptName="/ppp_on.sh"        # Название стартового скрипта в драйвере модема
restartScriptName="/ppp_restart.sh" # Название скрипта для перезапуска в драйвере модема, может совпадать с startScriptName

# Переменные для настройки
chargeTime=2             # Время зарядки конденсаторов перед включением в секундах
TimeToOn=1               # задержка после включения LDO в секундах
waitStatusTime=10        # Время ожидания готовности линии status в секундах
waitStatusAttempts=10	 # Макс. число циклов ожидания, каждый по $waitStatusTime секунд
pwrKeyTime=50000         # Время импульса pwr_key в микросекундах
waitPoweroffTime=3       # Время выключения модема
timeToReset=3            # Время удержания pwr_key pin для выключения модема в секундах
dirMode="0755"           # Права для destination LinkName
killPPPTime=2            # Время ожидания смерти pppd, необходимо для выполнения скриптов disconnect  и других процедур завершения pppd
waitPPPdTime=5           # Время ожидания завершения pppd при неправильном старте
waitPPPdAttempts=20	 # Макс. число циклов ожидания, каждый по $waitPPPdTime секунд
timePowerOffOn=3         # Время между выключением и включением модема

# Прочие переменные
hiLevel=1
lowLevel=0
lengthUsbId=9
exitCodeToReset=3
exitCodeToRestart=1
exitCodeOk=0

# Процедура выключения модема

reset(){

    echo "++++ Turning off the modem ++++"

    killall pppd
    sleep $killPPPTime

    gpioset --mode=time --sec=$timeToReset $pwrKeyPin=1
    gpioset  $pwrKeyPin=0
    echo "PWR_key clicked"

    echo "Wait for turning off"
    sleep $waitPoweroffTime

     # устанавливаем уровни по умолчанию
    echo "Set default level on pins:"
    echo "The GSM_ON  pin is set to level $lowLevel"
    gpioset $gsmOnPin=$lowLevel
    echo "The GSM_CHG pin is set to level $hiLevel"
    gpioset $chargePin=$hiLevel

    echo "Delete all files from  $parentDirectory$distanationLinkName"
    rm -rf $parentDirectory$distanationLinkName
    sleep $timePowerOffOn
    echo "The modem is turned off"
}

# Процедура запуска основного скрипта в драйвере модема
# если запускаемый файл существует, то запускается
# после запуска возвращается код завершения скрипта, иначе возвращается exitCodeToReset

startPPPdScript(){

    if [ ! -f $parentDirectory$distanationLinkName$startScriptName ]; then

        echo "Failed to find $parentDirectory$distanationLinkName$startScriptName to start pppd"
        return $exitCodeToReset
    fi

    $parentDirectory$distanationLinkName$startScriptName

    return $?
}

# Процедура перезапуска основного скрипта в драйвере модема
# если запускаемый файл существует, то запускается
# иначе запускается startScriptName
# возвращается код завершения скрипта

restartPPPdScript(){

    if [ ! -f $parentDirectory$distanationLinkName$restartScriptName ]; then

        echo "Failed to find $parentDirectory$distanationLinkName$restartScriptName to restart pppd"
        echo "Try exec $parentDirectory$distanationLinkName$startScriptName"
        startPPPdScript
        return $?
    fi

    $parentDirectory$distanationLinkName$restartScriptName
    return $?
}

# Процедура поиска модема с помощью lsusb
# $1 - имя переменной для передачи из процедуры vendorID:productID
# В случае успеха возвращается vendorID_productID в переменной с именем из $1

foundModem() {
    local  returnValue=$1
    eval  $returnValue=0
    local  ID=$(lsusb)

    if [ $? -ne 0 ]; then
        echo "lsusb error"
        reset
        exit $exitCodeToReset
    fi
    ID=$(echo "$ID" | awk -v pattern=$pattern '{if ($0 ~ pattern) {gsub(":","_");print $6; exit 0} }')
    if [ -z "$ID" ]; then
        echo "USB modem not found"
        return $exitCodeToReset
    fi
    if [ ${#ID} -ne $lengthUsbId ]; then
        echo "Failed to get Vendor|Product of USB device ID=$ID"
        return $exitCodeToReset
    fi
    echo "Found USB device $ID"
    eval $returnValue="'$ID'"
}

# Процедура создания ссылки на папку драйвера
# $1 = vendorID_productID (входной параметр)
# Если линк был ранее создан возвращаем exitCodeOk
# при ошибках создания ссылки возвращаем exitCodeToReset

createLink(){

    if [ ${#1} -ne $lengthUsbId ]; then

        echo "Failed to set Vendor|Product of USB device ID=$1"
        return $exitCodeToReset

    fi

    if [ -d $parentDirectory$distanationLinkName ]; then

        echo "Link found"
        return $exitCodeOk

    fi

    mkdir --mode=$dirMode -p  $parentDirectory

    if [ ! -d $parentDirectory ]; then

        echo "Failed to create directory $parentDirectory"
        return $exitCodeToReset

    fi

    ln -s -T $driversDirectory/$1 $parentDirectory$distanationLinkName

    if [ $? -ne 0 ]; then

        echo "Failed to run ln -s -T $driversDirectory/$1 $parentDirectory$distanationLinkName"
        return $exitCodeToReset

    fi

    if [ ! -d $parentDirectory$distanationLinkName ]; then

        echo "Failed to create link to $driversDirectory/$1 "
        return $exitCodeToReset

    fi

    echo "Link to the modem driver has been created"
    return $exitCodeOk
}

# Процедура проверки правильности имен пинов
# $1 имя пина (входной параметр)
# $2 переменная пина (выходной параметр)
# в случае успеха возвращается переменная имя которой находиться в $2 с параметрами пина $1 иначе выходим из скрипта

testPin(){

    local pinName=$1
    local pin=$(gpiofind $pinName)

    if [ $? -eq 0 ]; then

        local  returnValue=$2
        eval $returnValue="'$pin'"

    else

        echo "$pinName pin not found" >&2
        exit $exitCodeToReset

    fi
}

# Процедура инициализация модема
# В случае успеха возвращаем exitCodeOk,
# иначе возвращаем exitCodeToReset

init() {
    local j	# aux. cycle counter

    # Проверяем есть ли pppd
    echo "check $pppdPidFile"
    if [ -f $pppdPidFile ]; then
        echo "Found $pppdPidFile file"
        exit $exitCodeOk
    fi

    foundModem modemID
    if [ ${#modemID} -eq $lengthUsbId ]; then
        echo "Modem found"
        if [ -d $parentDirectory$distanationLinkName ]; then
            echo "Link on $parentDirectory$distanationLinkName found"
            restartPPPdScript
	    
            if [ $? -eq $exitCodeToReset ]; then
                reset
                exit $exitCodeToReset
	    else
		for j in $(seq $waitPPPdAttempts); do
		    sleep $waitPPPdTime
		    if [ -f $pppdPidFile ]; then
			echo "Start pppd successful"
			exit $exitCodeOk
		    fi
		done
                echo "Failed to start pppd"
                exit $exitCodeToRestart
	    fi
        else
            echo "link not found"
            reset
            return $exitCodeToReset
        fi
    fi
    sleep $timePowerOffOn
    # устанавливаем уровни по умолчанию
    echo "Set default level on pins:"
    echo "The GSM_CHG pin is set to level $hiLevel"
    gpioset $chargePin=$hiLevel
    echo "The GSM_ON  pin is set to level $lowLevel"
    gpioset $gsmOnPin=$lowLevel
    echo "The PWR_KEY pin is set to level $lowLevel"
    gpioset $pwrKeyPin=$lowLevel
    echo "The GSM_RST pin is set to level $lowLevel"
    gpioset $gsmRstPin=$lowLevel

    # включаем зарядку конденсаторов
    gpioset $chargePin=0
    echo "Charge is starting"
    echo "Wait charge time $chargeTime seconds"
    sleep $chargeTime
    echo "Turn on the main power"
    gpioset $gsmOnPin=1
    sleep $TimeToOn
    gpioset --mode=time --usec=$pwrKeyTime $pwrKeyPin=1
    gpioset  $pwrKeyPin=0
    echo "PWR_key clicked"
    echo "Waiting for the modem to be ready"
    for j in $(seq $waitStatusAttempts); do
	statusLevel=$(gpioget $gsmStatusPin)
	[[ $j -lt 2 ]] && echo -n "GSM Status level is $statusLevel" || echo -n "$statusLevel"
	# задержка после gpioget, но до принятия решения, чтобы udev успел "увидеть" устройство
	sleep $waitStatusTime
	[[ $statusLevel -eq 0 ]] && echo && break	# success
	echo -n "..."
    done
    if [ $statusLevel -ne 0 ]; then
        echo "Modem don't ready!"
        reset
        exit $exitCodeToReset
    fi

    echo "Modem initialization is finished"
    foundModem modemID
    if [ ${#modemID} -ne $lengthUsbId ]; then
        echo "Modem not found"
        echo "Turning off the power on the modem pins"
        reset
        exit $exitCodeToReset
    fi
    echo "Modem found"
    createLink $modemID
    if [ $? -ne 0 ]; then
        echo "Failed to create symbol link for modem's driver"
        reset
        exit $exitCodeToReset
    fi

    startPPPdScript

    if [ $? -eq $exitCodeToReset ]; then
        reset
        exit $exitCodeToReset
    else
	for j in $(seq $waitPPPdAttempts); do
	    sleep $waitPPPdTime
	    [[ -f $pppdPidFile ]] && break # success
	done
	if [ ! -f $pppdPidFile ]; then
	    echo "Failed to start pppd"
            reset
            exit $exitCodeToReset
	fi
    fi
    echo "Start pppd was successful"
}

# проверяем правильность пинов из DTS
testPin $chargePinName chargePin
testPin $gsmOnPinName gsmOnPin
testPin $pwrkeyPinName pwrKeyPin
testPin $gsmRstPinName gsmRstPin
testPin $gsmStatusPinName gsmStatusPin

case "$1" in

    start)
        echo "++++ Start $0 ++++"
        reset
        init

    ;;
    stop)
        echo "++++ Stop $0 ++++"
        reset

    ;;
    restart)
        echo "++++ Restart $0 ++++"
        reset
        init

    ;;
    kill)

        killall pppd

    ;;
    *)

        echo "Usage: $0 {start|stop|restart}"
        exit $exitCodeToRestart

    ;;
esac

exit $exitCodeOk
